package org.xesstandard.extensions;

import java.net.URI;
import java.util.List;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.attributes.XesListAttribute;
import org.xesstandard.model.attributes.XesStringAttribute;
import org.xesstandard.model.attributes.impl.XesListAttributeImpl;
import org.xesstandard.model.attributes.impl.XesStringAttributeImpl;
import org.xesstandard.model.impl.XesExtensionImpl;

public class XesArtifactLifecycleExtension extends XesExtensionImpl {

	public final static String PREFIX = "artifactlifecycle";

	private final static XesArtifactLifecycleExtension singleton = new XesArtifactLifecycleExtension();

	private static enum AttributeLevel {
		EVENT, META
	};

	/**
	 * Types of all defined attributes.
	 */
	private static enum AttributeType {
		LIST, STRING
	};

	/**
	 * All defined attributes.
	 */
	private static enum DefinedAttribute {
		MOVES("moves", AttributeLevel.EVENT, AttributeType.LIST, "Lifecycle Moves"), //
		MODEL("model", AttributeLevel.META, AttributeType.STRING, "Lifecycle Model"), //
		INSTANCE("instance", AttributeLevel.META, AttributeType.STRING, "Artifact Instance"), //
		TRANSITION("transition", AttributeLevel.META, AttributeType.STRING, "Lifecycle Transition");

		String key;
		String alias;
		AttributeLevel level;
		AttributeType type;

		DefinedAttribute(String key, AttributeLevel level, AttributeType type, String alias) {
			this.key = PREFIX + ":" + key;
			this.level = level;
			this.type = type;
			this.alias = alias;
		}
	}

	private XesArtifactLifecycleExtension() {
		super(true);
		setName("ArtifactLifecycle");
		setPrefix(PREFIX);
		try {
			setURI(new URI("http://www.xes-standard.org/swevent.xesext"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

		for (DefinedAttribute definedAttribute : DefinedAttribute.values()) {
			XesAttribute attribute;
			switch (definedAttribute.type) {
				case LIST : {
					attribute = new XesListAttributeImpl();
					break;
				}
				case STRING : {
					attribute = new XesStringAttributeImpl();
					break;
				}
				default : {
					attribute = null;
				}
			}
			attribute.setKey(definedAttribute.key);
			switch (definedAttribute.level) {
				case EVENT : {
					addEventAttribute(attribute);
					break;
				}
				case META : {
					addMetaAttribute(attribute);
					break;
				}
				default : {
				}
			}
			addAlias(attribute, "EN", definedAttribute.alias);
		}
	}

	/**
	 * Global key place holders. Can be initialized immediately.
	 */
	public static final String MOVES = DefinedAttribute.MOVES.key;
	public static final String MODEL = DefinedAttribute.MODEL.key;
	public static final String INSTANCE = DefinedAttribute.INSTANCE.key;
	public static final String TRANSITION = DefinedAttribute.TRANSITION.key;

	/**
	 * @return The single instance of this extension.
	 */
	public static XesArtifactLifecycleExtension instance() {
		return singleton;
	}

	private XesStringAttribute setString(XesComponent component, String key, String value) {
		XesStringAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesListAttribute setList(XesComponent component, String key) {
		XesListAttribute attribute = new XesListAttributeImpl();
		attribute.setKey(key);
		component.addAttribute(attribute);
		return attribute;
	}

	/*
	 * A list of handy assign and extract methods. Most are really
	 * straightforward.
	 */
	public String getInstance(XesAttribute attribute) {
		return attribute.getString(INSTANCE);
	}

	public XesStringAttribute setInstance(XesAttribute attribute, String instance) {
		return setString(attribute, INSTANCE, instance);
	}

	public String getTransition(XesAttribute attribute) {
		return attribute.getString(TRANSITION);
	}

	public XesStringAttribute setTransition(XesAttribute attribute, String transition) {
		return setString(attribute, TRANSITION, transition);
	}

	public List<XesAttribute> getMoves(XesEvent event) {
		return event.getList(MOVES);
	}

	/*
	 * Initializes the list of models attributes in the values element to
	 * the empty list. Use addModelValue to add model attributes to this
	 * list.
	 */
	public XesListAttribute setMoves(XesEvent event) {
		return setList(event, MOVES);
	}

	/*
	 * The exception to the rule. As model is a direct child attribute of
	 * the moves element, there is not parent attribute to have a set or get
	 * on. The models need to be retrieved by calling getMoves on the
	 * event, which returns the list of all model attributes as contained
	 * in the values element.
	 */
	public XesStringAttribute addModel(List<XesAttribute> values, String modelValue) {
		XesStringAttribute paramValueAttribute = new XesStringAttributeImpl();
		paramValueAttribute.setKey(MODEL);
		paramValueAttribute.setValue(modelValue);
		values.add(paramValueAttribute);
		return paramValueAttribute;
	}

}
