package org.xesstandard.extensions;

import java.net.URI;
import java.util.List;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.XesTrace;
import org.xesstandard.model.attributes.XesListAttribute;
import org.xesstandard.model.attributes.XesRealNumberAttribute;
import org.xesstandard.model.attributes.XesStringAttribute;
import org.xesstandard.model.attributes.impl.XesListAttributeImpl;
import org.xesstandard.model.attributes.impl.XesRealNumberAttributeImpl;
import org.xesstandard.model.attributes.impl.XesStringAttributeImpl;
import org.xesstandard.model.impl.XesExtensionImpl;

public class XesCostExtension extends XesExtensionImpl {

	public final static String TOTAL = "cost:total";
	public final static String CURRENCY = "cost:currency";
	public final static String DRIVERS = "cost:drivers";
	public final static String AMOUNT = "cost:amount";
	public final static String DRIVER = "cost:driver";
	public final static String TYPE = "cost:type";

	private final static XesCostExtension singleton = new XesCostExtension();

	private XesCostExtension() {
		super(true);
		setName("Cost");
		setPrefix("cost");
		try {
			setURI(new URI("http://www.xes-standard.org/cost.xesext"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

		XesAttribute attribute = new XesRealNumberAttributeImpl();
		attribute.setKey(TOTAL);
		addTraceAttribute(attribute);
		addEventAttribute(attribute);
		addAlias(attribute, "EN", "Total Cost");

		attribute = new XesStringAttributeImpl();
		attribute.setKey(CURRENCY);
		addTraceAttribute(attribute);
		addEventAttribute(attribute);
		addAlias(attribute, "EN", "Currency of Cost");

		attribute = new XesListAttributeImpl();
		attribute.setKey(DRIVERS);
		addTraceAttribute(attribute);
		addEventAttribute(attribute);
		addAlias(attribute, "EN", "Cost Drivers");

		attribute = new XesRealNumberAttributeImpl();
		attribute.setKey(AMOUNT);
		addMetaAttribute(attribute);
		addAlias(attribute, "EN", "Cost Amount");

		attribute = new XesStringAttributeImpl();
		attribute.setKey(DRIVER);
		addMetaAttribute(attribute);
		addAlias(attribute, "EN", "Cost Driver");

		attribute = new XesStringAttributeImpl();
		attribute.setKey(TYPE);
		addMetaAttribute(attribute);
		addAlias(attribute, "EN", "Cost Type");
	}

	/**
	 * @return The single instance of this extension.
	 */
	public static XesCostExtension instance() {
		return singleton;
	}

	private XesStringAttribute setString(XesComponent component, String key, String value) {
		XesStringAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesRealNumberAttribute setRealNumber(XesComponent component, String key, Double value) {
		XesRealNumberAttribute attribute = new XesRealNumberAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesListAttribute setList(XesComponent component, String key) {
		XesListAttribute attribute = new XesListAttributeImpl();
		attribute.setKey(key);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesStringAttribute addStringValue(XesListAttribute list, String key, String value) {
		XesStringAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		list.addValue(attribute);
		return attribute;
	}

	private XesRealNumberAttribute addRealNumberValue(XesListAttribute list, String key, Double value) {
		XesRealNumberAttribute attribute = new XesRealNumberAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		list.addValue(attribute);
		return attribute;
	}

	/**
	 * Returns the total costs of the given trace, that is, the value of the
	 * attribute with key "cost:total".
	 * 
	 * @param trace
	 *            The given trace.
	 * @return The total costs of the given trace.
	 */
	public Double getTotal(XesTrace trace) {
		return trace.getRealNumber(TOTAL);
	}

	/**
	 * Sets the total costs for the given trace, that is, the value of the
	 * attribute with key "cost:total".
	 * 
	 * @param trace
	 *            The given trace.
	 * @param value
	 *            The total costs to set.
	 * @return The attribute containing the total costs.
	 */
	public XesRealNumberAttribute setTotal(XesTrace trace, Double value) {
		return setRealNumber(trace, TOTAL, value);
	}

	/**
	 * Returns the total costs of the given event, that is, the value of the
	 * attribute with key "cost:total".
	 * 
	 * @param event
	 *            The given event.
	 * @return The total costs of the given event.
	 */
	public Double getTotal(XesEvent event) {
		return event.getRealNumber(TOTAL);
	}

	/**
	 * Sets the total costs for the given event, that is, the value of the
	 * attribute with key "cost:total".
	 * 
	 * @param event
	 *            The given event.
	 * @param value
	 *            The total costs to set.
	 * @return The attribute containing the total costs.
	 */
	public XesRealNumberAttribute setTotal(XesEvent event, Double value) {
		return setRealNumber(event, TOTAL, value);
	}

	/**
	 * Returns the currency of the given trace, that is, the value of the
	 * attribute with key "cost:currency".
	 * 
	 * @param trace
	 *            The given trace.
	 * @return The total costs of the given trace.
	 */
	public String getCurrency(XesTrace trace) {
		return trace.getString(CURRENCY);
	}

	/**
	 * Sets the currency for the given trace, that is, the value of the
	 * attribute with key "cost:currency".
	 * 
	 * @param trace
	 *            The given trace.
	 * @param value
	 *            The currency to set.
	 * @return The attribute containing the currency.
	 */
	public XesStringAttribute setCurrency(XesTrace trace, String value) {
		return setString(trace, CURRENCY, value);
	}

	/**
	 * Returns the currency of the given event, that is, the value of the
	 * attribute with key "cost:currency".
	 * 
	 * @param event
	 *            The given event.
	 * @return The total costs of the given event.
	 */
	public String getCurrency(XesEvent event) {
		return event.getString(CURRENCY);
	}

	/**
	 * Sets the currency for the given event, that is, the value of the
	 * attribute with key "cost:currency".
	 * 
	 * @param event
	 *            The given event.
	 * @param value
	 *            The currency to set.
	 * @return The attribute containing the currency.
	 */
	public XesStringAttribute setCurrency(XesEvent event, String value) {
		return setString(event, CURRENCY, value);
	}

	/**
	 * Returns the drivers of the given trace, that is, the value of the
	 * attribute with key "cost:drivers".
	 * 
	 * @param trace
	 *            The given trace.
	 * @return The drivers of the given trace.
	 */
	public List<XesAttribute> getDrivers(XesTrace trace) {
		return trace.getList(DRIVERS);
	}

	/**
	 * Initializes the drivers for the given trace, that is, the value of the attribute
	 * with key "cost:drivers".
	 * 
	 * @param trace
	 *            The given trace.
	 * @return The list attribute containing the drivers.
	 */
	public XesListAttribute setDrivers(XesTrace trace) {
		return setList(trace, DRIVERS);
	}

	/**
	 * Returns the drivers of the given event, that is, the value of the
	 * attribute with key "cost:drivers".
	 * 
	 * @param event
	 *            The given event.
	 * @return The drivers of the given event.
	 */
	public List<XesAttribute> getDrivers(XesEvent event) {
		return event.getList(DRIVERS);
	}

	/**
	 * Initializes the drivers for the given event, that is, the value of the attribute
	 * with key "cost:drivers".
	 * 
	 * @param event
	 *            The given event.
	 * @return The list attribute containing the drivers.
	 */
	public XesListAttribute setDrivers(XesEvent event) {
		return setList(event, DRIVERS);
	}

	/**
	 * Returns the amount of the given driver, that is, the value of the
	 * attribute with key "cost:amount".
	 * 
	 * @param attribute
	 *            The given driver.
	 * @return The amount of the given driver.
	 */
	public Double getAmount(XesAttribute attribute) {
		return attribute.getRealNumber(AMOUNT);
	}

	/**
	 * Sets the amount for the given driver, that is, the value of the attribute
	 * with key "cost:amount".
	 * 
	 * @param attribute
	 *            The given driver.
	 * @param value
	 *            The amount to set.
	 * @return The attribute containing the amount.
	 */
	public XesRealNumberAttribute setAmount(XesAttribute attribute, Double value) {
		return setRealNumber(attribute, AMOUNT, value);
	}

	/**
	 * Adds a driver to the list of drivers.
	 * 
	 * @param list
	 *            The list of drivers.
	 * @param value
	 *            The value for the new driver.
	 * @return The attribute containing the added driver.
	 */
	public XesStringAttribute addDriver(XesListAttribute list, String value) {
		return addStringValue(list, DRIVER, value);
	}

	/**
	 * Returns the type of the given driver, that is, the value of the attribute
	 * with key "cost:type".
	 * 
	 * @param attribute
	 *            The given driver.
	 * @return The type of the given driver.
	 */
	public String getType(XesAttribute attribute) {
		return attribute.getString(TYPE);
	}

	/**
	 * Sets the type for the given driver, that is, the value of the attribute
	 * with key "cost:type".
	 * 
	 * @param attribute
	 *            The given driver.
	 * @param value
	 *            The type to set.
	 * @return The attribute containing the type.
	 */
	public XesStringAttribute setType(XesAttribute attribute, String value) {
		return setString(attribute, TYPE, value);
	}
}
