package org.xesstandard.extensions;

import java.net.URI;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.attributes.XesIntegerNumberAttribute;
import org.xesstandard.model.attributes.XesStringAttribute;
import org.xesstandard.model.attributes.impl.XesIntegerNumberAttributeImpl;
import org.xesstandard.model.attributes.impl.XesStringAttributeImpl;
import org.xesstandard.model.impl.XesExtensionImpl;

public class XesSoftwareCommunicationExtension extends XesExtensionImpl {

	public final static String PREFIX = "swcomm";

	private final static XesSoftwareCommunicationExtension singleton = new XesSoftwareCommunicationExtension();

	private static enum AttributeLevel {
		EVENT
	};

	/**
	 * Types of all defined attributes.
	 */
	private static enum AttributeType {
		INT, STRING
	};

	/**
	 * All defined attributes.
	 */
	private static enum DefinedAttribute {
		LOCAL_HOST("localHost", AttributeLevel.EVENT, AttributeType.STRING, "Local endpoint - host name"), //
		LOCAL_PORT("localPort", AttributeLevel.EVENT, AttributeType.INT, "Local endpoint - port"), //
		REMOTE_HOST("remoteHost", AttributeLevel.EVENT, AttributeType.STRING, "Remote endpoint - host name"), //
		REMOTE_PORT("remotePort", AttributeLevel.EVENT, AttributeType.INT, "Remote endpoint - port");

		String key;
		String alias;
		AttributeLevel level;
		AttributeType type;

		DefinedAttribute(String key, AttributeLevel level, AttributeType type, String alias) {
			this.key = PREFIX + ":" + key;
			this.level = level;
			this.type = type;
			this.alias = alias;
		}
	}

	private XesSoftwareCommunicationExtension() {
		super(true);
		setName("Software Communication");
		setPrefix(PREFIX);
		try {
			setURI(new URI("http://www.xes-standard.org/swcomm.xesext"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

		for (DefinedAttribute definedAttribute : DefinedAttribute.values()) {
			XesAttribute attribute;
			switch (definedAttribute.type) {
				case INT : {
					attribute = new XesIntegerNumberAttributeImpl();
					break;
				}
				case STRING : {
					attribute = new XesStringAttributeImpl();
					break;
				}
				default : {
					attribute = null;
				}
			}
			attribute.setKey(definedAttribute.key);
			switch (definedAttribute.level) {
				case EVENT : {
					addEventAttribute(attribute);
					break;
				}
				default : {
				}
			}
			addAlias(attribute, "EN", definedAttribute.alias);
		}
	}

	/**
	 * Global key place holders. Can be initialized immediately.
	 */
	public static final String LOCAL_HOST = DefinedAttribute.LOCAL_HOST.key;
	public static final String LOCAL_PORT = DefinedAttribute.LOCAL_PORT.key;
	public static final String REMOTE_HOST = DefinedAttribute.REMOTE_HOST.key;
	public static final String REMOTE_PORT = DefinedAttribute.REMOTE_PORT.key;

	/**
	 * @return The single instance of this extension.
	 */
	public static XesSoftwareCommunicationExtension instance() {
		return singleton;
	}

	private XesStringAttribute setString(XesComponent component, String key, String value) {
		XesStringAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesIntegerNumberAttribute setIntegerNumber(XesComponent component, String key, Long value) {
		XesIntegerNumberAttribute attribute = new XesIntegerNumberAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	/*
	 * A list of handy assign and extract methods. Most are really
	 * straightforward.
	 */
	public String getLocalHost(XesEvent event) {
		return event.getString(LOCAL_HOST);
	}

	public XesStringAttribute setLocalHost(XesEvent event, String localHost) {
		return setString(event, LOCAL_HOST, localHost);
	}

	public long getLocalPort(XesEvent event) {
		return event.getIntegerNumber(LOCAL_PORT);
	}

	public XesIntegerNumberAttribute setLocalPort(XesEvent event, long localPort) {
		return setIntegerNumber(event, LOCAL_PORT, localPort);
	}

	public String getRemoteHost(XesEvent event) {
		return event.getString(REMOTE_HOST);
	}

	public XesStringAttribute setRemoteHost(XesEvent event, String remoteHost) {
		return setString(event, REMOTE_HOST, remoteHost);
	}

	public long getRemotePort(XesEvent event) {
		return event.getIntegerNumber(REMOTE_PORT);
	}

	public XesIntegerNumberAttribute setRemotePort(XesEvent event, long remotePort) {
		return setIntegerNumber(event, REMOTE_PORT, remotePort);
	}

}
