package org.xesstandard.extensions;

import java.net.URI;
import java.util.List;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.XesLog;
import org.xesstandard.model.attributes.XesBooleanAttribute;
import org.xesstandard.model.attributes.XesIntegerNumberAttribute;
import org.xesstandard.model.attributes.XesListAttribute;
import org.xesstandard.model.attributes.XesStringAttribute;
import org.xesstandard.model.attributes.impl.XesBooleanAttributeImpl;
import org.xesstandard.model.attributes.impl.XesIntegerNumberAttributeImpl;
import org.xesstandard.model.attributes.impl.XesListAttributeImpl;
import org.xesstandard.model.attributes.impl.XesStringAttributeImpl;
import org.xesstandard.model.impl.XesExtensionImpl;

public class XesSoftwareEventExtension extends XesExtensionImpl {

	public final static String PREFIX = "swevent";

	/**
	 * Possible software event types
	 */
	public enum SoftwareEventType {
		CALL("call", "The start of a method block"), //
		CALLING("calling", "The start of calling / invoking another method"), //
		HANDLE("handle", "The start of an exception handle catch block"), //
		RETURN("return", "The normal end of a method block"), //
		RETURNING("returning", "The end of returning a called method"), //
		THROWS("throws", "The end of a method block in case of an uncaught exception"), //
		UNKNOWN("unknown", "Unknown software event type");

		private final String encoding;

		private SoftwareEventType(String encoding, String description) {
			this.encoding = encoding;
		}

		public String getEncoding() {
			return encoding;
		}

		public static SoftwareEventType decode(String encoding) {
			encoding = encoding.trim().toLowerCase();
			for (SoftwareEventType transition : SoftwareEventType.values()) {
				if (transition.encoding.equals(encoding)) {
					return transition;
				}
			}
			return SoftwareEventType.UNKNOWN;
		}

	}

	private final static XesSoftwareEventExtension singleton = new XesSoftwareEventExtension();

	private static enum AttributeLevel {
		EVENT, LOG, META
	};

	/**
	 * Types of all defined attributes.
	 */
	private static enum AttributeType {
		BOOLEAN, INT, LIST, STRING
	};

	/**
	 * All defined attributes.
	 */
	private static enum DefinedAttribute {
		APP_NAME("appName", AttributeLevel.EVENT, AttributeType.STRING, "User defined application tier"), //
		APP_NODE("appNode", AttributeLevel.EVENT, AttributeType.STRING, "User defined application node"), //
		APP_SESSION("appSession", AttributeLevel.EVENT, AttributeType.STRING, "User defined application session"), //
		APP_TIER("appTier", AttributeLevel.EVENT, AttributeType.STRING, "User defined application name"), //
		CALLEE_CLASS("callee-class", AttributeLevel.EVENT, AttributeType.STRING, "Callee - Class"), //
		CALLEE_FILENAME("callee-filename", AttributeLevel.EVENT, AttributeType.STRING,
				"Callee - File name source code artifact"), //
		CALLEE_INSTANCEID("callee-instanceId", AttributeLevel.EVENT, AttributeType.STRING,
				"Callee - Instance id of class instance"), //
		CALLEE_ISCONSTRUCTOR("callee-isConstructor", AttributeLevel.EVENT, AttributeType.BOOLEAN,
				"Callee - Is a class constructor"), //
		CALLEE_LINENR("callee-lineNr", AttributeLevel.EVENT, AttributeType.INT,
				"Callee - Line number in source code artifact"), //
		CALLEE_METHOD("callee-method", AttributeLevel.EVENT, AttributeType.STRING, "Callee - Method"), //
		CALLEE_PACKAGE("callee-package", AttributeLevel.EVENT, AttributeType.STRING, "Callee - Package"), //
		CALLEE_PARAMSIG("callee-paramSig", AttributeLevel.EVENT, AttributeType.STRING, "Callee - Parameter signature"), //
		CALLEE_RETURNSIG("callee-returnSig", AttributeLevel.EVENT, AttributeType.STRING, "Callee - Return signature"), //
		CALLER_CLASS("caller-class", AttributeLevel.EVENT, AttributeType.STRING, "Caller - Class"), //
		CALLER_FILENAME("caller-filename", AttributeLevel.EVENT, AttributeType.STRING,
				"Caller - File name source code artifact"), //
		CALLER_INSTANCEID("caller-instanceId", AttributeLevel.EVENT, AttributeType.STRING,
				"Caller - Instance id of class instance"), //
		CALLER_ISCONSTRUCTOR("caller-isConstructor", AttributeLevel.EVENT, AttributeType.BOOLEAN,
				"Caller - Is a class constructor"), //
		CALLER_LINENR("caller-lineNr", AttributeLevel.EVENT, AttributeType.INT,
				"Caller - Line number in source code artifact"), //
		CALLER_METHOD("caller-method", AttributeLevel.EVENT, AttributeType.STRING, "Caller - Method"), //
		CALLER_PACKAGE("caller-package", AttributeLevel.EVENT, AttributeType.STRING, "Caller - Package"), //
		CALLER_PARAMSIG("caller-paramSig", AttributeLevel.EVENT, AttributeType.STRING, "Caller - Parameter signature"), //
		CALLER_RETURNSIG("caller-returnSig", AttributeLevel.EVENT, AttributeType.STRING, "Caller - Return signature"), //
		EX_CAUGHT("exCaught", AttributeLevel.EVENT, AttributeType.STRING, "Caught exception type"), //
		EX_THROWN("exThrown", AttributeLevel.EVENT, AttributeType.STRING, "Thrown exception type"), //
		HAS_DATA("hasData", AttributeLevel.LOG, AttributeType.BOOLEAN, "Has method data"), //
		HAS_EXCEPTION("hasException", AttributeLevel.LOG, AttributeType.BOOLEAN, "Has exception data"), //
		NANOTIME("nanotime", AttributeLevel.EVENT, AttributeType.INT, "Elapsed nano time"), //
		PARAMS("params", AttributeLevel.EVENT, AttributeType.LIST, "List of parameters for the called method"), //
		PARAM_VALUE("paramValue", AttributeLevel.META, AttributeType.STRING, "A parameter value in the list params"), //
		RETURN_VALUE("returnValue", AttributeLevel.EVENT, AttributeType.STRING, "Return value for the returning method"), //
		THREAD_ID("threadId", AttributeLevel.EVENT, AttributeType.STRING, "Thread id for generated event"), //
		TYPE("type", AttributeLevel.EVENT, AttributeType.STRING, "Event type"), //
		VALUE_TYPE("valueType", AttributeLevel.META, AttributeType.STRING,
				"A runtime value type for a return or parameter value");

		String key;
		String alias;
		AttributeLevel level;
		AttributeType type;

		DefinedAttribute(String key, AttributeLevel level, AttributeType type, String alias) {
			this.key = PREFIX + ":" + key;
			this.level = level;
			this.type = type;
			this.alias = alias;
		}
	}

	private XesSoftwareEventExtension() {
		super(true);
		setName("SoftwareEvent");
		setPrefix(PREFIX);
		try {
			setURI(new URI("http://www.xes-standard.org/swevent.xesext"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

		for (DefinedAttribute definedAttribute : DefinedAttribute.values()) {
			XesAttribute attribute;
			switch (definedAttribute.type) {
				case BOOLEAN : {
					attribute = new XesBooleanAttributeImpl();
					break;
				}
				case INT : {
					attribute = new XesIntegerNumberAttributeImpl();
					break;
				}
				case LIST : {
					attribute = new XesListAttributeImpl();
					break;
				}
				case STRING : {
					attribute = new XesStringAttributeImpl();
					break;
				}
				default : {
					attribute = null;
				}
			}
			attribute.setKey(definedAttribute.key);
			switch (definedAttribute.level) {
				case LOG : {
					addLogAttribute(attribute);
					break;
				}
				case EVENT : {
					addEventAttribute(attribute);
					break;
				}
				case META : {
					addMetaAttribute(attribute);
					break;
				}
				default : {
				}
			}
			addAlias(attribute, "EN", definedAttribute.alias);
		}
	}

	/**
	 * Global key place holders. Can be initialized immediately.
	 */
	public static final String APP_NAME = DefinedAttribute.APP_NAME.key;
	public static final String APP_NODE = DefinedAttribute.APP_NODE.key;
	public static final String APP_SESSION = DefinedAttribute.APP_SESSION.key;
	public static final String APP_TIER = DefinedAttribute.APP_TIER.key;
	public static final String CALLEE_CLASS = DefinedAttribute.CALLEE_CLASS.key;
	public static final String CALLEE_FILENAME = DefinedAttribute.CALLEE_FILENAME.key;
	public static final String CALLEE_INSTANCEID = DefinedAttribute.CALLEE_INSTANCEID.key;
	public static final String CALLEE_ISCONSTRUCTOR = DefinedAttribute.CALLEE_ISCONSTRUCTOR.key;
	public static final String CALLEE_LINENR = DefinedAttribute.CALLEE_LINENR.key;
	public static final String CALLEE_METHOD = DefinedAttribute.CALLEE_METHOD.key;
	public static final String CALLEE_PACKAGE = DefinedAttribute.CALLEE_PACKAGE.key;
	public static final String CALLEE_PARAMSIG = DefinedAttribute.CALLEE_PARAMSIG.key;
	public static final String CALLEE_RETURNSIG = DefinedAttribute.CALLEE_RETURNSIG.key;
	public static final String CALLER_CLASS = DefinedAttribute.CALLER_CLASS.key;
	public static final String CALLER_FILENAME = DefinedAttribute.CALLER_FILENAME.key;
	public static final String CALLER_INSTANCEID = DefinedAttribute.CALLER_INSTANCEID.key;
	public static final String CALLER_ISCONSTRUCTOR = DefinedAttribute.CALLER_ISCONSTRUCTOR.key;
	public static final String CALLER_LINENR = DefinedAttribute.CALLER_LINENR.key;
	public static final String CALLER_METHOD = DefinedAttribute.CALLER_METHOD.key;
	public static final String CALLER_PACKAGE = DefinedAttribute.CALLER_PACKAGE.key;
	public static final String CALLER_PARAMSIG = DefinedAttribute.CALLER_PARAMSIG.key;
	public static final String CALLER_RETURNSIG = DefinedAttribute.CALLER_RETURNSIG.key;
	public static final String EX_CAUGHT = DefinedAttribute.EX_CAUGHT.key;
	public static final String EX_THROWN = DefinedAttribute.EX_THROWN.key;
	public static final String HAS_DATA = DefinedAttribute.HAS_DATA.key;
	public static final String HAS_EXCEPTION = DefinedAttribute.HAS_EXCEPTION.key;
	public static final String NANOTIME = DefinedAttribute.NANOTIME.key;
	public static final String PARAMS = DefinedAttribute.PARAMS.key;
	public static final String PARAM_VALUE = DefinedAttribute.PARAM_VALUE.key;
	public static final String RETURN_VALUE = DefinedAttribute.RETURN_VALUE.key;
	public static final String THREAD_ID = DefinedAttribute.THREAD_ID.key;
	public static final String TYPE = DefinedAttribute.TYPE.key;
	public static final String VALUE_TYPE = DefinedAttribute.VALUE_TYPE.key;

	/**
	 * @return The single instance of this extension.
	 */
	public static XesSoftwareEventExtension instance() {
		return singleton;
	}

	private XesStringAttribute setString(XesComponent component, String key, String value) {
		XesStringAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesIntegerNumberAttribute setIntegerNumber(XesComponent component, String key, Long value) {
		XesIntegerNumberAttribute attribute = new XesIntegerNumberAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesBooleanAttribute setBoolean(XesComponent component, String key, Boolean value) {
		XesBooleanAttribute attribute = new XesBooleanAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesListAttribute setList(XesComponent component, String key) {
		XesListAttribute attribute = new XesListAttributeImpl();
		attribute.setKey(key);
		component.addAttribute(attribute);
		return attribute;
	}

	/*
	 * A list of handy assign and extract methods. Most are really
	 * straightforward.
	 */
	public String getAppName(XesEvent event) {
		return event.getString(APP_NAME);
	}

	public XesStringAttribute setAppName(XesEvent event, String appName) {
		return setString(event, APP_NAME, appName);
	}

	public String getAppNode(XesEvent event) {
		return event.getString(APP_NODE);
	}

	public XesStringAttribute setAppNode(XesEvent event, String appNode) {
		return setString(event, APP_NODE, appNode);
	}

	public String getAppSession(XesEvent event) {
		return event.getString(APP_SESSION);
	}

	public XesStringAttribute setAppSession(XesEvent event, String appSession) {
		return setString(event, APP_SESSION, appSession);
	}

	public String getAppTier(XesEvent event) {
		return event.getString(APP_TIER);
	}

	public XesStringAttribute setAppTier(XesEvent event, String appTier) {
		return setString(event, APP_TIER, appTier);
	}

	public String getCalleeClass(XesEvent event) {
		return event.getString(CALLEE_CLASS);
	}

	public XesStringAttribute setCalleeClass(XesEvent event, String calleeClass) {
		return setString(event, CALLEE_CLASS, calleeClass);
	}

	public String getCalleeFilename(XesEvent event) {
		return event.getString(CALLEE_FILENAME);
	}

	public XesStringAttribute setCalleeFilename(XesEvent event, String calleeFilename) {
		return setString(event, CALLEE_FILENAME, calleeFilename);
	}

	public String getCalleeInstanceId(XesEvent event) {
		return event.getString(CALLEE_INSTANCEID);
	}

	public XesStringAttribute setCalleeInstanceId(XesEvent event, String calleeInstanceId) {
		return setString(event, CALLEE_INSTANCEID, calleeInstanceId);
	}

	public boolean getCalleeIsConstructor(XesEvent event) {
		return event.getBoolean(CALLEE_ISCONSTRUCTOR);
	}

	public XesBooleanAttribute setCalleeIsConstructor(XesEvent event, boolean CalleeIsConstructor) {
		return setBoolean(event, CALLEE_ISCONSTRUCTOR, CalleeIsConstructor);
	}

	public long getCalleeLineNr(XesEvent event) {
		return event.getIntegerNumber(CALLEE_LINENR);
	}

	public XesIntegerNumberAttribute setCalleeLineNr(XesEvent event, long calleeLineNr) {
		return setIntegerNumber(event, CALLEE_LINENR, calleeLineNr);
	}

	public String getCalleeMethod(XesEvent event) {
		return event.getString(CALLEE_METHOD);
	}

	public XesStringAttribute setCalleeMethod(XesEvent event, String calleeMethod) {
		return setString(event, CALLEE_METHOD, calleeMethod);
	}

	public String getCalleePackage(XesEvent event) {
		return event.getString(CALLEE_PACKAGE);
	}

	public XesStringAttribute setCalleePackage(XesEvent event, String calleePackage) {
		return setString(event, CALLEE_PACKAGE, calleePackage);
	}

	public String getCalleeParamSig(XesEvent event) {
		return event.getString(CALLEE_PARAMSIG);
	}

	public XesStringAttribute setCalleeParamSig(XesEvent event, String calleeParamSig) {
		return setString(event, CALLEE_PARAMSIG, calleeParamSig);
	}

	public String getCalleeReturnSig(XesEvent event) {
		return event.getString(CALLEE_RETURNSIG);
	}

	public XesStringAttribute setCalleeReturnSig(XesEvent event, String calleeReturnSig) {
		return setString(event, CALLEE_RETURNSIG, calleeReturnSig);
	}

	public String getCallerClass(XesEvent event) {
		return event.getString(CALLER_CLASS);
	}

	public XesStringAttribute setCallerClass(XesEvent event, String callerClass) {
		return setString(event, CALLER_CLASS, callerClass);
	}

	public String getCallerFilename(XesEvent event) {
		return event.getString(CALLER_FILENAME);
	}

	public XesStringAttribute setCallerFilename(XesEvent event, String callerFilename) {
		return setString(event, CALLER_FILENAME, callerFilename);
	}

	public String getCallerInstanceId(XesEvent event) {
		return event.getString(CALLER_INSTANCEID);
	}

	public XesStringAttribute setCallerInstanceId(XesEvent event, String callerInstanceId) {
		return setString(event, CALLER_INSTANCEID, callerInstanceId);
	}

	public boolean getCallerIsConstructor(XesEvent event) {
		return event.getBoolean(CALLER_ISCONSTRUCTOR);
	}

	public XesBooleanAttribute setCallerIsConstructor(XesEvent event, boolean CalleeIsConstructor) {
		return setBoolean(event, CALLER_ISCONSTRUCTOR, CalleeIsConstructor);
	}

	public long getCallerLineNr(XesEvent event) {
		return event.getIntegerNumber(CALLER_LINENR);
	}

	public XesIntegerNumberAttribute setCallerLineNr(XesEvent event, long callerLineNr) {
		return setIntegerNumber(event, CALLER_LINENR, callerLineNr);
	}

	public String getCallerMethod(XesEvent event) {
		return event.getString(CALLER_METHOD);
	}

	public XesStringAttribute setCallerMethod(XesEvent event, String callerMethod) {
		return setString(event, CALLER_METHOD, callerMethod);
	}

	public String getCallerPackage(XesEvent event) {
		return event.getString(CALLER_PACKAGE);
	}

	public XesStringAttribute setCallerPackage(XesEvent event, String callerPackage) {
		return setString(event, CALLER_PACKAGE, callerPackage);
	}

	public String getCallerParamSig(XesEvent event) {
		return event.getString(CALLER_PARAMSIG);
	}

	public XesStringAttribute setCallerParamSig(XesEvent event, String callerParamSig) {
		return setString(event, CALLER_PARAMSIG, callerParamSig);
	}

	public String getCallerReturnSig(XesEvent event) {
		return event.getString(CALLER_RETURNSIG);
	}

	public XesStringAttribute setCallerReturnSig(XesEvent event, String callerReturnSig) {
		return setString(event, CALLER_RETURNSIG, callerReturnSig);
	}

	public String getExCaught(XesEvent event) {
		return event.getString(EX_CAUGHT);
	}

	public XesStringAttribute setExCaught(XesEvent event, String exCaught) {
		return setString(event, EX_CAUGHT, exCaught);
	}

	public String getExThrown(XesEvent event) {
		return event.getString(EX_THROWN);
	}

	public XesStringAttribute setExThrown(XesEvent event, String exThrown) {
		return setString(event, EX_THROWN, exThrown);
	}

	public boolean getHasData(XesLog log) {
		return log.getBoolean(HAS_DATA);
	}

	public XesBooleanAttribute setHasData(XesLog log, boolean hasData) {
		return setBoolean(log, HAS_DATA, hasData);
	}

	public boolean getHasException(XesLog log) {
		return log.getBoolean(HAS_EXCEPTION);
	}

	public XesBooleanAttribute setHasException(XesLog log, boolean hasException) {
		return setBoolean(log, HAS_EXCEPTION, hasException);
	}

	public long getNanotime(XesEvent event) {
		return event.getIntegerNumber(NANOTIME);
	}

	public XesIntegerNumberAttribute setNanotime(XesEvent event, long nanotime) {
		return setIntegerNumber(event, NANOTIME, nanotime);
	}

	public List<XesAttribute> getParams(XesEvent event) {
		return event.getList(PARAMS);
	}

	/*
	 * Initializes the list of paramValue attributes in the values element to
	 * the empty list. Use addParamValue to add paramValue attributes to this
	 * list.
	 */
	public XesListAttribute setParams(XesEvent event) {
		return setList(event, PARAMS);
	}

	/*
	 * The exception to the rule. As paramValue is a direct child attribute of
	 * the values element, there is not parent attribute to have a set or get
	 * on. The parameter values need to be retrieved by calling getParams on the
	 * event, which returns the list of all paramValue attributes as contained
	 * in the values element.
	 */
	public XesStringAttribute addParamValue(List<XesAttribute> values, String paramValue) {
		XesStringAttribute paramValueAttribute = new XesStringAttributeImpl();
		paramValueAttribute.setKey(PARAM_VALUE);
		paramValueAttribute.setValue(paramValue);
		values.add(paramValueAttribute);
		return paramValueAttribute;
	}

	public String getReturnValue(XesEvent event) {
		return event.getString(RETURN_VALUE);
	}

	public XesStringAttribute setReturnValue(XesEvent event, String returnValue) {
		return setString(event, RETURN_VALUE, returnValue);
	}

	public String getThreadId(XesEvent event) {
		return event.getString(THREAD_ID);
	}

	public XesStringAttribute setThreadId(XesEvent event, String threadId) {
		return setString(event, THREAD_ID, threadId);
	}

	public SoftwareEventType getType(XesEvent event) {
		return SoftwareEventType.decode(event.getString(TYPE));
	}

	public XesStringAttribute setType(XesEvent event, SoftwareEventType type) {
		return setString(event, TYPE, type.encoding);
	}

	public String getValueType(XesAttribute attribute) {
		return attribute.getString(VALUE_TYPE);
	}

	public XesStringAttribute setValueType(XesAttribute attribute, String valueType) {
		return setString(attribute, VALUE_TYPE, valueType);
	}

}