package org.xesstandard.extensions;

import java.net.URI;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.attributes.XesIntegerNumberAttribute;
import org.xesstandard.model.attributes.XesRealNumberAttribute;
import org.xesstandard.model.attributes.impl.XesIntegerNumberAttributeImpl;
import org.xesstandard.model.attributes.impl.XesRealNumberAttributeImpl;
import org.xesstandard.model.impl.XesExtensionImpl;

public class XesSoftwareTelemetryExtension extends XesExtensionImpl {

	public final static String PREFIX = "swtelemetry";

	private final static XesSoftwareTelemetryExtension singleton = new XesSoftwareTelemetryExtension();

	private static enum AttributeLevel {
		EVENT
	};

	/**
	 * Types of all defined attributes.
	 */
	private static enum AttributeType {
		INT, FLOAT
	};

	/**
	 * All defined attributes.
	 */
	private static enum DefinedAttribute {
		CPU_TOTAL_USER("cpuTotalUser", AttributeLevel.EVENT, AttributeType.INT,
				"CPU usage - total time in user space, in milliseconds"), //
		CPU_TOTAL_KERNEL("cpuTotalKernel", AttributeLevel.EVENT,
				AttributeType.INT,
				"CPU usage - total time in kernel space, in milliseconds"), //
		CPU_TOTAL_IDLE("cpuTotalIdle", AttributeLevel.EVENT, AttributeType.INT,
				"CPU usage - total time spent idle, in milliseconds"), //
		CPU_LOAD_USER("cpuLoadUser", AttributeLevel.EVENT, AttributeType.FLOAT,
				"CPU usage - load in user space"), //
		CPU_LOAD_KERNEL("cpuLoadKernel", AttributeLevel.EVENT,
				AttributeType.FLOAT, "CPU usage - load in kernel space"), //
		THREAD_TOTAL("threadTotal", AttributeLevel.EVENT, AttributeType.INT,
				"Total number of threads"), //
		THREAD_DAEMON("threadDaemon", AttributeLevel.EVENT, AttributeType.INT,
				"Number of daemon threads"), //
		MEMORY_USED("memoryUsed", AttributeLevel.EVENT, AttributeType.INT,
				"Total memory used, measured in bytes"), //
		MEMORY_TOTAL("memoryTotal", AttributeLevel.EVENT, AttributeType.INT,
				"Total memory available, measured in bytes"), //
		MEMORY_LOAD("memoryLoad", AttributeLevel.EVENT, AttributeType.FLOAT,
				"Memory usage load");

		String key;
		String alias;
		AttributeLevel level;
		AttributeType type;

		DefinedAttribute(String key, AttributeLevel level, AttributeType type, String alias) {
			this.key = PREFIX + ":" + key;
			this.level = level;
			this.type = type;
			this.alias = alias;
		}
	}
	
	private XesSoftwareTelemetryExtension() {
		super(true);
		setName("Software Telemetry");
		setPrefix(PREFIX);
		try {
			setURI(new URI("http://www.xes-standard.org/swtelemetry.xesext"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

		for (DefinedAttribute definedAttribute : DefinedAttribute.values()) {
			XesAttribute attribute;
			switch (definedAttribute.type) {
				case INT : {
					attribute = new XesIntegerNumberAttributeImpl();
					break;
				}
				case FLOAT : {
					attribute = new XesRealNumberAttributeImpl();
					break;
				}
				default : {
					attribute = null;
				}
			}
			attribute.setKey(definedAttribute.key);
			switch (definedAttribute.level) {
				case EVENT : {
					addEventAttribute(attribute);
					break;
				}
				default : {
				}
			}
			addAlias(attribute, "EN", definedAttribute.alias);
		}
	}

	/**
	 * Global key place holders. Can be initialized immediately.
	 */
	public static final String CPU_TOTAL_USER = DefinedAttribute.CPU_TOTAL_USER.key;
	public static final String CPU_TOTAL_KERNEL = DefinedAttribute.CPU_TOTAL_KERNEL.key;
	public static final String CPU_TOTAL_IDLE = DefinedAttribute.CPU_TOTAL_IDLE.key;
	public static final String CPU_LOAD_USER = DefinedAttribute.CPU_LOAD_USER.key;
	public static final String CPU_LOAD_KERNEL = DefinedAttribute.CPU_LOAD_KERNEL.key;
	public static final String THREAD_TOTAL = DefinedAttribute.THREAD_TOTAL.key;
	public static final String THREAD_DAEMON = DefinedAttribute.THREAD_DAEMON.key;
	public static final String MEMORY_USED = DefinedAttribute.MEMORY_USED.key;
	public static final String MEMORY_TOTAL = DefinedAttribute.MEMORY_TOTAL.key;
	public static final String MEMORY_LOAD = DefinedAttribute.MEMORY_LOAD.key;

	/**
	 * @return The single instance of this extension.
	 */
	public static XesSoftwareTelemetryExtension instance() {
		return singleton;
	}

	private XesRealNumberAttribute setRealNumber(XesComponent component, String key, Double value) {
		XesRealNumberAttribute attribute = new XesRealNumberAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	private XesIntegerNumberAttribute setIntegerNumber(XesComponent component, String key, Long value) {
		XesIntegerNumberAttribute attribute = new XesIntegerNumberAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}

	/*
	 * A list of handy assign and extract methods. Most are really
	 * straightforward.
	 */
	public long getCPUTotalUser(XesEvent event) {
		return event.getIntegerNumber(CPU_TOTAL_USER);
	}

	public XesIntegerNumberAttribute setCPUTotalUser(XesEvent event, long cpuTotalUser) {
		return setIntegerNumber(event, CPU_TOTAL_USER, cpuTotalUser);
	}

	public long getCPUTotalKernel(XesEvent event) {
		return event.getIntegerNumber(CPU_TOTAL_KERNEL);
	}

	public XesIntegerNumberAttribute setCPUTotalKernel(XesEvent event, long cpuTotalKernel) {
		return setIntegerNumber(event, CPU_TOTAL_KERNEL, cpuTotalKernel);
	}

	public long getCPUTotalIdle(XesEvent event) {
		return event.getIntegerNumber(CPU_TOTAL_IDLE);
	}

	public XesIntegerNumberAttribute setCPUTotalIdle(XesEvent event, long cpuTotalIdle) {
		return setIntegerNumber(event, CPU_TOTAL_IDLE, cpuTotalIdle);
	}

	public double getCPULoadUser(XesEvent event) {
		return event.getRealNumber(CPU_LOAD_USER);
	}

	public XesRealNumberAttribute setCPULoadUser(XesEvent event, double cpuLoadUser) {
		return setRealNumber(event, CPU_LOAD_USER, cpuLoadUser);
	}

	public double getCPULoadKernel(XesEvent event) {
		return event.getRealNumber(CPU_LOAD_KERNEL);
	}

	public XesRealNumberAttribute setCPULoadKernel(XesEvent event, double cpuLoadKernel) {
		return setRealNumber(event, CPU_LOAD_KERNEL, cpuLoadKernel);
	}

	public long getThreadTotal(XesEvent event) {
		return event.getIntegerNumber(THREAD_TOTAL);
	}

	public XesIntegerNumberAttribute setThreadTotal(XesEvent event, long threadTotal) {
		return setIntegerNumber(event, THREAD_TOTAL, threadTotal);
	}

	public long getThreadDaemon(XesEvent event) {
		return event.getIntegerNumber(THREAD_DAEMON);
	}

	public XesIntegerNumberAttribute setThreadDaemon(XesEvent event, long threadDaemon) {
		return setIntegerNumber(event, THREAD_DAEMON, threadDaemon);
	}

	public long getMemoryUsed(XesEvent event) {
		return event.getIntegerNumber(MEMORY_USED);
	}

	public XesIntegerNumberAttribute setMemoryUsed(XesEvent event, long memoryUsed) {
		return setIntegerNumber(event, MEMORY_USED, memoryUsed);
	}

	public long getMemoryTotal(XesEvent event) {
		return event.getIntegerNumber(MEMORY_TOTAL);
	}

	public XesIntegerNumberAttribute setMemoryTotal(XesEvent event, long memoryTotal) {
		return setIntegerNumber(event, MEMORY_TOTAL, memoryTotal);
	}

	public double getMemoryLoad(XesEvent event) {
		return event.getRealNumber(MEMORY_LOAD);
	}

	public XesRealNumberAttribute setMemoryLoad(XesEvent event, double memoryLoad) {
		return setRealNumber(event, MEMORY_LOAD, memoryLoad);
	}

}