package org.xesstandard.xml;

import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesClassifier;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.XesExtension;
import org.xesstandard.model.XesLog;
import org.xesstandard.model.XesTrace;
import org.xesstandard.model.attributes.XesListAttribute;

public class XesXmlWriterStax implements XesXmlWriter {

	public String getDescription() {
		return "Writes an IEEE XES log to a plain XML serialization using StAX";
	}

	public String getName() {
		return "IEEE XES XML (using StAX)";
	}

	public String getAuthor() {
		return "Eric Verbeek";
	}

	public String[] getSuffices() {
		return new String[] { "xes" };
	}

	public void write(XesLog log, OutputStream output) {
		XMLOutputFactory factory = XMLOutputFactory.newInstance();
		try {
			XMLStreamWriter writer = factory.createXMLStreamWriter(output, StandardCharsets.UTF_8.name());

			writer.writeStartDocument(StandardCharsets.UTF_8.name(), "1.0");
			
			if (log != null) {
				write(log, writer);
			}
			writer.writeEndDocument();

			writer.flush();
			writer.close();

		} catch (XMLStreamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void write(XesLog log, XMLStreamWriter writer) throws XMLStreamException {
		writer.writeStartElement(XesXmlTags.LOG);

		writeAttribute(writer, XesXmlTags.XESVERSION, log.getVersion());
		writeAttributeList(writer, XesXmlTags.XESFEATURES, log.getFeatures());

		for (XesExtension extension : log.getExtensions()) {
			write(extension, writer);
		}

		if (!log.getGlobalTraceAttributes().isEmpty()) {
			writer.writeStartElement(XesXmlTags.GLOBAL);
			writeAttribute(writer, XesXmlTags.SCOPE, XesXmlTags.TRACE);
			for (XesAttribute globalTraceAttribute : log.getGlobalTraceAttributes()) {
				write(globalTraceAttribute, writer);
			}
			writer.writeEndElement();
		}

		if (!log.getGlobalEventAttributes().isEmpty()) {
			writer.writeStartElement(XesXmlTags.GLOBAL);
			writeAttribute(writer, XesXmlTags.SCOPE, XesXmlTags.EVENT);
			for (XesAttribute globalEventAttribute : log.getGlobalEventAttributes()) {
				write(globalEventAttribute, writer);
			}
			writer.writeEndElement();
		}

		for (XesClassifier traceClassifier : log.getTraceClassifiers()) {
			write(traceClassifier, writer, XesXmlTags.TRACE);
		}

		for (XesClassifier eventClassifier : log.getEventClassifiers()) {
			write(eventClassifier, writer, XesXmlTags.EVENT);
		}
		
		for (XesAttribute attribute : log.getAttributes()) {
			write(attribute, writer);
		}
		
		for (XesTrace trace : log.getTraces()) {
			write(trace, writer);
		}
		
		for (XesEvent event : log.getEvents()) {
			write(event, writer);
		}
		
		writer.writeEndElement();
	}

	private void write(XesExtension extension, XMLStreamWriter writer) throws XMLStreamException {
		writer.writeStartElement(XesXmlTags.EXTENSION);

		writeAttribute(writer, XesXmlTags.NAME, extension.getName());
		writeAttribute(writer, XesXmlTags.PREFIX, extension.getPrefix());
		writeAttribute(writer, XesXmlTags.URI, extension.getURI());

		writer.writeEndElement();
	}

	private void write(XesAttribute attribute, XMLStreamWriter writer) throws XMLStreamException {
		switch (attribute.getType()) {
			case STRING :
				writer.writeStartElement(XesXmlTags.STRING);
				break;
			case BOOLEAN :
				writer.writeStartElement(XesXmlTags.BOOLEAN);
				break;
			case DATETIME :
				writer.writeStartElement(XesXmlTags.DATE);
				break;
			case ID :
				writer.writeStartElement(XesXmlTags.ID);
				break;
			case INTEGERNUMBER :
				writer.writeStartElement(XesXmlTags.INT);
				break;
			case REALNUMBER :
				writer.writeStartElement(XesXmlTags.FLOAT);
				break;
			case LIST :
				writer.writeStartElement(XesXmlTags.STRING);
				break;
			default :
				return;
		}

		writeAttribute(writer, XesXmlTags.KEY, attribute.getKey());

		if (attribute.isSimple()) {
			writeAttribute(writer, XesXmlTags.VALUE, attribute.getStringValue());
		}

		for (XesAttribute metaAttribute : attribute.getAttributes()) {
			write(metaAttribute, writer);
		}

		if (!attribute.isSimple()) {
			writer.writeStartElement(XesXmlTags.VALUES);
			for (XesAttribute valueAttribute : ((XesListAttribute) attribute).getValues()) {
				write(valueAttribute, writer);
			}
			writer.writeEndElement();
		}
		writer.writeEndElement();
	}

	private void write(XesClassifier classifier, XMLStreamWriter writer, String scope) throws XMLStreamException {
		writer.writeStartElement(XesXmlTags.CLASSIFIER);
		
		writeAttribute(writer, XesXmlTags.NAME, classifier.getName());
		writeAttribute(writer, XesXmlTags.SCOPE, scope);
		String keys = "";
		String div = "";
		for (String key : classifier.getKeys()) {
			keys += div + key;
			div = " ";
		}
		writeAttribute(writer, XesXmlTags.KEYS, keys);
		
		writer.writeEndElement();
	}
	
	private void write(XesTrace trace, XMLStreamWriter writer) throws XMLStreamException {
		writer.writeStartElement(XesXmlTags.TRACE);

		for (XesAttribute attribute : trace.getAttributes()) {
			write(attribute, writer);
		}

		for (XesEvent event : trace.getEvents()) {
			write(event, writer);
		}

		writer.writeEndElement();
	}
	
	private void write(XesEvent event, XMLStreamWriter writer) throws XMLStreamException {
		writer.writeStartElement(XesXmlTags.EVENT);

		for (XesAttribute attribute : event.getAttributes()) {
			write(attribute, writer);
		}

		writer.writeEndElement();
	}
	
	private void writeAttribute(XMLStreamWriter writer, String key, Object value) throws XMLStreamException {
		if (value != null) {
			writer.writeAttribute(key, value.toString());
		}
	}

	private void writeAttributeList(XMLStreamWriter writer, String key, List<String> values) throws XMLStreamException {
		String div = "";
		String valueList = "";
		for (String value : values) {
			if (value != null) {
				valueList += div + value;
				div = " ";
			}
		}
		writer.writeAttribute(key, valueList);
	}
}
